#include <stdio.h>
#include <ctype.h>

//    for (int i = 0; text[i] != 0; ++i) {
//        if (isalpha(text[i])) {
//            return &text[i];
//        }
//    }

/**
 * Πλήρης (τρίτη) εκδοχή της συνάρτησης, όπου επιστρέφει μέσω call-by-reference, δηλαδή
 * του pointer L, το μήκος της λέξης που βρέθηκε. Εάν δεν βρεθεί λέξη, εκτός από NULL
 * τότε βάζουμε και μηδενικό μήκος στην L.
 *
 * ΠΡΟΣΟΧΗ! Η L είναι προαιρετικό όρισμα, με την έννοια ότι μπορεί κάποιος να δώσει NULL
 * επειδή δεν τον ενδιαφέρει να πάρει το μήκος της λέξης
 */
char *nextWord( char *text , int *L ) {
    static char *oldText = NULL;
    if (text == NULL) {
        if (oldText == NULL) {
            // εάν δόθηκε η L... (ο έλεγχος αυτός πρέπει να γίνει αλλιώς το *L θα κάνει το πρόγραμμα να crashάρει)
            if (L != NULL) {
                // αν δεν υπάρχει προηγούμενο κείμενο, αφού δεν βρέθηκε λέξη,
                // βάζουμε και μηδενικό μήκος ως ένδειξη
                *L = 0;
            }
            return NULL;
        }

        text = oldText;

        while( isalpha(*text) && *text != 0 ) {
            text++;
        }
    }

    while( ! isalpha(*text) && *text != 0 ) {
        text++;
    }

    if (*text == 0) {
        // εάν δόθηκε η L... (ο έλεγχος αυτός πρέπει να γίνει αλλιώς το *L θα κάνει το πρόγραμμα να crashάρει)
        if (L != NULL) {
            // εάν δεν βρέθηκε λέξη μπορούμε να πούμε ότι το μήκος της είναι 0
            *L = 0;
        }
        text = NULL;
    }

    oldText = text;

    // εάν δόθηκε pointer σε ακέραιο και όχι NULL , αλλά υπάρχει και εφόσον όντως βρέθηκε λέξη (text != NULL)
    if (L != NULL && text != NULL) {
        // αρχικά μηδένισε αυτόν τον ακέραιο ώστε να μετρήσουμε τα γράμματα της λέξης
        *L = 0;

        // για κάθε αλφαβητικό χαρακτήρα και όσο είμαστε εντός κειμένου
        while (isalpha(*text) && *text != 0) {
            // προχωράμε στον επόμενο χαρακτήρα
            text++;
            // και αυξάνουμε κατά ένα το μήκος της λέξης, αφού βρήκαμε ένας ακόμα
            // χαρακτήρας πληρεί τις προϋποθέσεις της while αυτής
            (*L)++;
        }
    }

    // επειδή η τιμή του text ενδεχομένως άλλαξε για να προχωρήσει μέχρι το τέλος της λέξης
    // επιστρέφουμε την τιμή oldText που ήταν ίση με την text πριν την αλλαγή. Αλλιώς, αντί
    // για την αρχή της λέξης, θα επιστρέφαμε τον 1ο χαρακτήρα μετά από αυτή!
    return oldText;
}

/**
 * Δεύτερη εκδοχή της συνάρτησης, στην οποία εάν της δοθεί NULL τότε συνεχίζει στο
 * ίδιο κείμενο (στο σημείο που είχε μείνει) η προηγούμενη εκτέλεση της συνάρτησης.
 */
//char *nextWord( char *text ) {
//    // χρειαζόμαστε μία μεταβλητή που να "θυμάται" που είχαμε μείνει στην
//    // προηγούμενη εκτέλεση, άρα για να παραμένει η τιμή της θα πρέπει να
//    // είναι static. Αρχικά έχει την τιμή NULL επειδή δεν υπάρχει κάποια
//    // προηγούμενη εκτέλεση της συνάρτησης.
//    static char *oldText = NULL;
//
//    // Εάν το text είναι NULL, σημαίνει ότι πρέπει να συνεχίσουμε από εκεί
//    // που είχαμε μείνει στην προηγούμενη εκτέλεση της nextWord
//    if (text == NULL) {
//        // Εάν και το προηγούμενο κείμενο ήταν κενό (NULL)
//        if (oldText == NULL) {
//            // τότε δεν βρέθηκε κάτι
//            return NULL;
//        } // δεν χρησιμοποιούμε else (αν και θα μπορούσαμε) επειδή  στο if υπάρχει η return
//
//        // αλλιώς για κείμενο χρησιμοποίησε το "παλιό"
//        text = oldText;
//
//        // επειδή το κείμενο αυτό είναι σε αρχή λέξης, θα πρέπει πρώτα να προχωρήσουμε
//        // ώστε να "βγούμε" από τη λέξη
//        // όσο ο χαρακτήρας του κειμένο είναι αλφαβητικός (άρα είμαστε ακόμα μέσα στη λέξη)
//        // και δεν έχει τελειώσει και το κείμενο
//        while( isalpha(*text) && *text != 0 ) {
//            // συνέχισε με τον επόμενο χαρακτήρα
//            text++;
//        }
//    }
//
//    while( ! isalpha(*text) && *text != 0 ) {
//        text++;
//    }
//
//    if (*text == 0) {
//        // αντί να επιστρέψουμε άμεσα την τιμή NULL
//        // την αναθέτουμε το text, ώστε να μπει σωστά
//        // και στην oldText ακολούθως
//        text = NULL;
//    }
//
//    // τώρα που ολοκληρώθηκε η διαδικασία, απομνημονεύουμε
//    // στην oldText το σημείο στο οποίο φτάσαμε (δηλαδή
//    // την αρχή της λέξης που βρήκαμε ή NULL), ώστε να
//    // είναι έτοιμη η τιμή του oldText για επόμενη κλήση
//    // της συνάρτησης
//    oldText = text;
//
//    return text;
//}


/**
 * Πρώτη εκδοχή της συνάρτησης, η οποία επιστρέφει τον pointer στην 1η λέξη του κειμένου
 * ή NULL αν δεν βρεθεί λέξη. Ως λέξη πάντα θεωρούμε την ακολουθία συνεχόμενων αλφαβητικών
 * χαρακτήρων (αυτούς που δίνουν αληθές στην isalpha)
 *
 * Προσέξτε ότι σκοπίμως δεν εξετάζουμε την περίπτωση text == NULL στην αρχή, καθώς θα
 * ασχοληθούμε με αυτό στα επόμενα ερωτήματα.
 */
//char *nextWord( char *text ) {
//
//    // όσο (while) ο χαρακτήρας που "δείχνει" η text (δηλαδή ο *text) ΔΕΝ(!) είναι
//    // αλφαβητικός (isalpha) [άρα όσο δεν είμαστε μέσα σε λέξη], και ταυτόχρονα δεν
//    // φτάσαμε στο τέλους του κειμένου ( *text != 0 )
//    while( ! isalpha(*text) && *text != 0 ) {
//        // προχώρα στον επόμενο χαρακτήρα του κειμένου (text++)
//        text++;
//    }
//
//    // Εάν ψαχνοντας για λέξη φτάσαμε στο τέλος του κειμένου
//    if (*text == 0) {
//        // να επιστρέψεις NULL ώστε να δείξουμε ότι δεν βρέθηκε κάτι
//        return NULL;
//    }
//
//    // Εάν η εκτέλεση έφτασε μέχρι εδώ, σημαίνει ότι βρήκαμε μια λέξη
//    // οπότε επιστρέφουμε τον pointer στο σημείο που βρέθηκε αυτή η λέξη
//    // (το 1ο της γράμμα δηλαδή)
//    return text;
//}

int main() {
    char *myText = "   Kalh - hmera   ";
    int len1 = 1000;
    int len2 = 1000;
    char *First = nextWord(myText, &len1);
    char *Second = nextWord(NULL, &len2);
    printf("%s\n%s\n%s\n", myText, First, len1, Second, len2);
    return 0;
}
